//==============================================================================
//
//  リーフォニーのためのサンプルスケッチ
//
//     用途  : ビーコン・スキャナー（受信側）
//		 　　　　なるべく簡単に！
//     MCU   : STM32L452RE (Nucleo-64/Nucleo L452RE)
//     STM32 Core   : Ver1.9.0
//     Arduino IDEのバージョン  : 1.8.13
//
//     リーフの構成
//       (1) AI04 LCD				キャラクタLCD
//       (2) AC02 BLE Sugar	BLE ビーコンを発信
//       (3) AP03 STM32 MCU	MCU
//       (4) AZ01 USB				デバッグ（プログラム書き込み）用
//       (5) AV03 AA BAT		単３電池
//
//    元のプログラム : 
//    https://github.com/Leafony/Sample-Sketches/tree/master/STM32_4-Sensors_BLE
//    (c) 2021 LEAFONY SYSTEMS Co.
//
//
//    Released under the MIT license（ライセンス：MITライセンス）
//    https://opensource.org/licenses/mit-license.php//
//
//    Rev.1 2021/10/01 	modified by Yamagishi
//
//==============================================================================


//==============================================================================
// 定義
//==============================================================================

#include "TBGLib.h"                         // BLEライブラリ
#include <Wire.h>                           // I2C通信ライブラリ（LCDとの通信）
#include <ST7032.h>                         // LCDライブラリ


//------------------------------------------------------------------------------
// ビーコン受信時対象のデバイスを判別するためのデバイス名
// 長さは必ず6文字
//------------------------------------------------------------------------------
//                     |123456|
String strDeviceName = "DCUT4a";


//------------------------------------------------------------------------------
// シリアルコンソールへのデバック出力
//    #define DEBUG = 出力あり
//　　// #define DEBUG = 出力なし（コメントアウトする）
//------------------------------------------------------------------------------
#define DEBUG


//------------------------------------------------------------------------------
// IOピンの名前定義
// ＊：今回使用したピン
//------------------------------------------------------------------------------
//＊D0  PA3  (UART2_RXD)
//＊D1  PA2  (UART2_TXD)
//  D2  PC7  (INT0)
//  D3  PB3  (INT1)
//  D4  PB5
//  D5  PB4
//  D6  PA8
//＊D7  PB12
//  D8  PA9  (UART1_TX)
//  D9  PA10 (UART1_RX)
//  D10 PB6  (SS)
//  D11 PA7  (MOSI)
//  D12 PA6  (MISO)
//  D13 PA5  (SCK)
//＊D14 PB9  (SDA)
//＊D15 PB8  (SCL)
//  A0  PA4
//＊A1  PA0  (UART4_TX)
//＊A2  PA1  (UART4_RX)
//  A3  PB0
//  A4  PC1
//  A5  PC0
//------------------------------------------------------------------------------
#define BLE_WAKEUP      PB12
#define BLE_RX          PA1
#define BLE_TX          PA0


//------------------------------------------------------------------------------
// プログラム内で使用する定数定義
//------------------------------------------------------------------------------
//-----------------------------------------------
// LCD I2Cアドレス
//-----------------------------------------------
#define LCD_I2C_EXPANDER_ADDR   0x1A        // LCD I2C Expander


//==============================================================================
// オブジェクト
//==============================================================================

//------------------------------
// LCD
//------------------------------
  ST7032 lcd;

//------------------------------
// BLE
//------------------------------
	HardwareSerial Serialble(BLE_RX, BLE_TX);						// ハードウエアシリアル
	BGLib ble112((HardwareSerial *)&Serialble, 0, 0 );


//==============================================================================
// プログラム
//==============================================================================

//------------------------------------------------------------------------------
// I2C 通信
//------------------------------------------------------------------------------

//-----------------------------------------------
// I2C スレーブに１バイト書き込む
//-----------------------------------------------
void i2c_write_byte(int device_address, int reg_address, int write_data){

  Wire.beginTransmission(device_address);
  Wire.write(reg_address);
  Wire.write(write_data);
  Wire.endTransmission();

}

//-----------------------------------------------
// I2C スレーブから１バイト読み込む
//-----------------------------------------------
unsigned char i2c_read_byte(int device_address, int reg_address){

  int read_data = 0;

  Wire.beginTransmission(device_address);
  Wire.write(reg_address);
  Wire.endTransmission(false);

  Wire.requestFrom(device_address, 1);
  read_data = Wire.read();

  return read_data;

}


//------------------------------------------------------------------------------
// その他関数
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// 整数型データを送信文字列に変換
// param     : uint8_t *upperVal	upper byte 送信データ上位
//           : uint8_t *lowerVal	lower byte 送信データ下位
//           : uint16_t sourceVal 2bytes　送信したいデータ
//------------------------------------------------------------------------------
void convUint8fromInt16( uint8_t *upperVal, uint8_t *lowerVal, uint16_t sourceVal ){

  *upperVal = (sourceVal >> 8) & 0x0FF;
  *lowerVal = sourceVal & 0x0FF;

}

//------------------------------------------------------------------------------
// 受信文字列を整数型データに変換
// param     : uint8_t upperVal	upper byte 受信データ上位
//           : uint8_t lowerVal	lower byte 受信データ下位
// return    : int message value 4bytes 受信したデータ
// 注意：STM32のint型は 4byteなのでキャストしないと正しく入力できない
//------------------------------------------------------------------------------
int convInt16fromInt8( uint8_t upperVal, uint8_t lowerVal ){

  return (int16)((upperVal << 8) |(lowerVal & 0x0FF));

}
// Rem. 32-bit integer type is 4 bytes.


//------------------------------------------------------------------------------
// setup / 初期化
//------------------------------------------------------------------------------
void setup(){

  Serial.begin(115200);										// プログラム書き込み・デバッグ用通信
  delay(1000);														// 少し待つと安定する（MCUによってはこれが必要）
  Wire.begin();														// I2C 100KHz（LCD用）

	//---- LCD初期化 ----
  i2c_write_byte(LCD_I2C_EXPANDER_ADDR, 0x03, 0xFE);
  i2c_write_byte(LCD_I2C_EXPANDER_ADDR, 0x01, 0x01);      // LCD 	電源ON

  lcd.begin(8, 2);												// 初期表示
  lcd.setContrast(30);
  lcd.clear();
  lcd.print("Start");
  lcd.setCursor(0, 1);
  lcd.print("scan");

	//---- ポート初期化 ----
  pinMode(BLE_WAKEUP, OUTPUT);           // [D7] : BLE Wakeup/Sleep
  digitalWrite(BLE_WAKEUP, HIGH);        // BLE Wakeup

 
  //---- BLE初期化 ----

  // BGLib イベントハンドラの登録
  ble112.ble_evt_le_gap_scan_response = my_evt_le_gap_scan_response;
 
	// BLE用のUART設定
  Serialble.begin(9600);
 
	// ディスカバリモードの設定
  ble112.ble_cmd_le_gap_discover(2);
  while (ble112.checkActivity(1000));
  delay(1000);
 
}


//------------------------------------------------------------------------------
// loop / メインループ
//------------------------------------------------------------------------------
void loop() {

    ble112.checkActivity(0);                                    /* Receive check */

}


//------------------------------------------------------------------------------
// BLEのコールバック関数
// イベント発生時にライブラリから呼ばれる関数
//------------------------------------------------------------------------------

//----------------------------------------------
//スキャン応答
//----------------------------------------------
void my_evt_le_gap_scan_response(const struct ble_msg_le_gap_scan_response_evt_t  *msg ){

	// 受信データの構造
  // struct ble_msg_le_gap_scan_response_evt_t {
  //		int8        rssi;            [4]
  //  	uint8       packet_type;     [5]
  //  	bd_addr     sender;          [6~11]
  //  	uint8       address_type;    [12]
  //  	uint8       bond;            [13]
  //  	uint8array  data;            [14]
  //} __attribute__((packed));


  uint8 data_len = msg -> data.len;
  String rcv_data = "";

	//---- 受信データ文字列の長さを制限 ----
	if( data_len >30 ){			// 未知のデバイスからの長いデータでバッファ長を超える
		data_len =40;					// メモリを破壊して暴走する場合がある
	}

	#ifdef DEBUG
	//---- データ部分を抽出 ----
  for (uint8_t i = 1; i <= data_len; i++) {
    rcv_data += String(msg -> data.data[data_len - i], HEX);
  }
	#endif
 
	//---- 送信元アドレスを抽出 ----
  String rcv_sender = "";
  for (int i = 0; i < 6 ;i++){
    rcv_sender += String(msg->sender.addr[i],HEX);
  }

	//---- 送信元デバイス名を抽出 ----
  char rcv_name[7];
  for (int i = 0; i < 6 ;i++){
		rcv_name[i] = msg->data.data[i+5];
  }
		rcv_name[6] = 0;
	String s = rcv_name;


	//-------- 対象となるデバイスの時のみ --------
	if( !s.compareTo(strDeviceName) ){

		//---- 受信データを整数化 ----
		int rcv_cnt = convInt16fromInt8( msg -> data.data[15], msg -> data.data[16] );
		int rcv_axisX = convInt16fromInt8( msg -> data.data[17], msg -> data.data[18] );
		int rcv_axisY = convInt16fromInt8( msg -> data.data[19], msg -> data.data[20] );
		int rcv_axisZ = convInt16fromInt8( msg -> data.data[21], msg -> data.data[22] );

		//---- LCD表示 ----
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("X:"+String(rcv_axisX,DEC));
    lcd.setCursor(0, 1);
    lcd.print("Y:"+String(rcv_axisY,DEC));

		#ifdef DEBUG									// 受信した内容
	    Serial.print(rcv_cnt);
	    Serial.print(" / ");
	    Serial.print(rcv_axisX);
	    Serial.print(" / ");
	    Serial.print(rcv_axisY);
	    Serial.print(" / ");
	    Serial.print(rcv_axisZ);
	    Serial.print(" / ");
	
	    Serial.print("  oRSSI: ");
	    Serial.print(msg->rssi,DEC);
	    Serial.print( "  address: " );
	    Serial.print(rcv_sender);
	
	    Serial.print( "  rcv_cnt: " );
	    Serial.print(rcv_cnt);
	
	    Serial.print("  data: ");
	    Serial.print(rcv_data);
	    Serial.println();
		#endif

	}
}


//==============================================================================
// End of code
